'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _assert = require('assert');

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var fromSourceString = function fromSourceString(code) {
    return (0, _babelTemplate2.default)('(' + code + ')')().expression;
};

exports.default = function (_ref) {
    var _ref$types = _ref.types;
    var callExpression = _ref$types.callExpression;
    var identifier = _ref$types.identifier;
    var importDeclaration = _ref$types.importDeclaration;
    var importSpecifier = _ref$types.importSpecifier;
    var memberExpression = _ref$types.memberExpression;
    var stringLiteral = _ref$types.stringLiteral;
    var variableDeclaration = _ref$types.variableDeclaration;
    var variableDeclarator = _ref$types.variableDeclarator;
    var isImportDeclaration = _ref$types.isImportDeclaration;
    var isImportDefaultSpecifier = _ref$types.isImportDefaultSpecifier;
    var isImportNamespaceSpecifier = _ref$types.isImportNamespaceSpecifier;
    var isImportSpecifier = _ref$types.isImportSpecifier;
    var isStringLiteral = _ref$types.isStringLiteral;
    var assertImportDefaultSpecifier = _ref$types.assertImportDefaultSpecifier;

    var testOptions = function testOptions(opts, value, match) {
        var optsArray = Array.isArray(opts) ? opts : [opts];
        optsArray.some(function (_ref2) {
            var options = _ref2.options;
            var test = _ref2.test;

            if (test && new RegExp(test).test(value)) {
                return match(readFileSyncCallExpressionArgs(value, options));
            }
        });
    };
    var readFileSyncCallExpressionArgs = function readFileSyncCallExpressionArgs(value, options) {
        // [require.resolve(<file>)[, encoding|options]]
        var args = [callExpression(memberExpression(identifier('require'), identifier('resolve')), [stringLiteral(value)])];
        if (options) {
            // Second argument to readFileSync can be string or object
            args.push(fromSourceString(JSON.stringify(options)));
        }
        return args;
    };
    return {
        visitor: {
            ImportDeclaration: function ImportDeclaration(path, _ref3) {
                var opts = _ref3.opts;
                var _path$node = path.node;
                var specifiers = _path$node.specifiers;
                var value = _path$node.source.value;

                testOptions(opts, value, function (readFileSyncArgs) {
                    (0, _assert.equal)(specifiers.length, 1, 'Number of import specifiers.');
                    assertImportDefaultSpecifier(specifiers[0]);
                    var local = specifiers[0].local;
                    // Search import <specifiers> 'fs';

                    var readFileSync = void 0;
                    var fsImportDeclarations = path.parent.body.filter(isImportDeclaration).filter(function (_ref4) {
                        var value = _ref4.source.value;
                        return value === 'fs';
                    });
                    if (!fsImportDeclarations.some(function (_ref5) {
                        var specifiers = _ref5.specifiers;
                        return specifiers.find(function (specifier) {
                            if (isImportDefaultSpecifier(specifier) || isImportNamespaceSpecifier(specifier)) {
                                readFileSync = memberExpression(specifier.local, identifier('readFileSync'));
                                return true;
                            }
                            if (isImportSpecifier(specifier)) {
                                if (specifier.imported.name === 'readFileSync') {
                                    readFileSync = specifier.local;
                                    return true;
                                }
                            }
                        });
                    })) {
                        // fs is not imported
                        readFileSync = path.scope.generateUidIdentifier('readFileSync');
                        // Add import { readFileSync as _readFileSync } from 'fs';
                        path.insertBefore(importDeclaration([importSpecifier(readFileSync, identifier('readFileSync'))], stringLiteral('fs')));
                    }
                    // const <localName> = <readFileSync>(<args>);
                    path.replaceWith(variableDeclaration('const', [variableDeclarator(local, callExpression(readFileSync, readFileSyncArgs))]));
                    return true;
                });
            },
            CallExpression: function CallExpression(path, _ref6) {
                var opts = _ref6.opts;

                if (path.node.callee.name === 'require') {
                    var args = path.node.arguments;
                    if (args.length) {
                        var arg = args[0];
                        if (isStringLiteral(arg)) {
                            testOptions(opts, arg.value, function (readFileSyncArgs) {
                                // require('fs').readFileSync(<args>)
                                path.replaceWith(callExpression(memberExpression(callExpression(identifier('require'), [stringLiteral('fs')]), identifier('readFileSync')), readFileSyncArgs));
                                return true;
                            });
                        }
                    }
                }
            }
        }
    };
};